Page  60,132
Title ATTCLKFX - AT&T PC 6300 Software Clock Fix and Display
;
; ATTCLKFX - AT&T PC 6300 Software Clock Fix and Display
;
;  +------------+----------------------------------------------------+
;  |  Command   |  Parameters                                        |
;  +------------+----------------------------------------------------+
;  |            |                                                    |
;  |  ATTCLKFX  |  [ FIXONLY ]             [ FACTOR(nn) ]            |
;  |            |                                                    |
;  |            |  [ CHIME / NOCHIME ]     [ FREQ(nnnn) ]            |
;  |            |                                                    |
;  |            |  [ DATE / NODATE ]       [ TIME / NOTIME ]         |
;  |            |                                                    |
;  |            |  [ hotkey ]                                        |
;  |            |                                                    |
;  |            |  [ DISPLAY(fore back) ]  [ WINDOW(fore back) ]     |
;  |            |                                                    |
;  +------------+----------------------------------------------------+
;
; The software clock maintained in the BIOS data area at location
; 0040H:006CH (t_low_order) and 0040H:006EH (t_hi_order) is updated
; 18.75 times each second on the AT&T PC 6300. In terms of minutes
; this equates to 1125 updates per minute and in terms of hours this
; equates to 67500 updates per hour.
;
; This presents the following problem:
;
;      The software clock counters, t_low_order and t_hi_order are
;      wrap around counters, that is, when t_low_order reaches a
;      value of 65535 it wraps back to 0 and t_hi_order is incremented
;      by 1 on the next RTC interrupt. This wrap around is intended
;      to signify the changing of the hour. However, on the AT&T PC
;      6300 the hour change occurs at 67500 as can be seen in the
;      following formula:
;
;      18.75 * 60 * 60 = 67500
;                 
;                 minutes per hour
;             seconds per minute
;      RTC interrupts per second
;
;      Software such as the Right Hand Man by Red E Products, Inc.
;      and the Reminder Alarm published in the June 23, 1987,
;      Volume 6 Number 12, issue of PC Magazine utilize these
;      counters as follows:
;
;      t_low_order - number of RTC interrupts during the current
;                    hour. Dividing the counter by 1093 results in
;                    a value equivalent to the current minute within
;                    the hour.
;
;      t_hi_order  - the number of hours elapsed since midnight.
;
;      This is acceptable on an IBM PC because the RTC interrupts
;      occur 18.21 times a second not 18.75 times a second as on
;      the AT&T PC 6300. I say it is acceptable because it still
;      does not provide an accurate timing. When these counters are
;      used as a clock on the IBM PC the clock will gain approximately
;      1 to 2.5 seconds per hour.
;
; Another problem related to the software clock is a bug in the AT&T
; provided MS-DOS 3.1 (may also exist in AT&T MS-DOS 2.11). The
; software clock is not set from the hardware clock during a system
; boot. Instead the software clock represents the time since the last
; system boot. This problem also exists when PC-DOS 3.1 is used with
; the clock driver CLOCKDEV.SYS which may be found on bulletin boards
; as CLK_AT&T.ARC. This problem was corrected in AT&T MS-DOS 3.2 R1.02,
; however, this version of DOS seems to lose 20-30 seconds between
; boots for some unknown reason.
;
; ATTCLKFX.COM is intended to correct these problems. ATTCLKFX performs
; the following functions:
;
; 1. Ensures the BIOS data area software clock is set to the current
;    time.
;
; 2. Makes itself resident intercepting the Real Time Clock (08H),
;    Keyboard (16H) and Timer (1CH) interrupts.
;
; The resident portion of ATTCLKFX performs the following functions:
;
; Real Time Clock Interrupt
;
;   1. Approximates an RTC interrupt rate of 18.20 per second by
;      skipping every nnth (default: 35) RTC interrupt resulting in
;      1092 (default) interrupts per minute and 65520 interrupts per
;      hour.
;
;   2. Forces a wrap to occur when the t_lo_order counter has reached
;      65519 by changing the counter value to 65535.
;
; Keyboard Interrupt
;
;   1. Displays the ATTCLKFX window.
;
;   2. Allows the clock speed to be adjusted (faster, slower).
;
;   3. Allows the date and time display and hourly chime to be turned
;      on and off.
;
; Timer Interrupt
;
;   1. Displays the current Date and Time in the upper right corner of
;      the screen.
;
;   2. Sounds the hourly chime.
          Page
.xall
.xcref
;--------------------------- InLine Macros -----------------------------
Display   Macro  string
          ifdif  <string>,<DX>
          ifdif  <string>,<dx>
          Lea    DX, string             ; Message to Display
          endif
          endif
          Mov    AH, 09H                ; Display
          Int    21H                    ;   String
          EndM

Get_Interrupt Macro num
          Mov    AL, num                ; Interrupt Number
          Mov    AH, 35H                ; Get
          Int    21H                    ;   Interrupt Vector
          EndM

Keep_Process Macro rc,addr
          Lea    DX, addr               ; End of Resident Routine
          Add    DX, 15                 ; Compute Number
          Mov    CL, 4                  ;   of Paragraphs
          Shr    DX, CL                 ;     Rounded Up
          ifdif  <rc>,<AL>
          ifdif  <rc>,<al>
          Mov    AL, rc                 ; Return Code
          endif
          endif
          Mov    AH, 31H                ; Terminate
          Int    21H                    ;   and Stay Resident
          EndM

Param     Macro  param1,param2,param3
          local  local1,local2,local3,local4
          ifdif  <&param1>,<EntryMap>
          ifdif  <&param1>,<TableEnd>
local1    DW     local2                 ; Entry Length
          DW     param3                 ; Routine Address
          DW     param2                 ; Parameter Value
          DW     local4                 ; Parameter Length
local3    DB     '&param1'              ; Parameter
local4    Equ    $-local3
local2    Equ    $-local1
          endif
          endif
          ifidn  <&param1>,<EntryMap>
.lall
Entry_Len Equ    $-$                    ; Entry Length
Entry_Rtn Equ    Entry_Len+2            ; Routine Address
Entry_Wrd Equ    Entry_Rtn+2            ; Parameter Value
Entry_Lo  Equ    Entry_Wrd              ; Value Low  Byte
Entry_Hi  Equ    Entry_Lo+1             ; Value High Byte
Entry_PLn Equ    Entry_Hi+1             ; Parameter Length
Entry_Prm Equ    Entry_PLn+2            ; Parameter
.xall
          endif
          ifidn  <&param1>,<TableEnd>
          DB     0FFH                   ; Table End
          endif
          EndM

PopEm     Macro  r1,r2,r3,r4,r5,r6,r7,r8,r9
          IFNB   <r1>
          Pop    r1                     ; Restore r1
          PopEm  r2,r3,r4,r5,r6,r7,r8,r9
          ENDIF
          EndM

PushEm    Macro  r1,r2,r3,r4,r5,r6,r7,r8,r9
          IFNB   <r1>
          Push   r1                     ; Save r1
          PushEm r2,r3,r4,r5,r6,r7,r8,r9
          ENDIF
          EndM

Set_Interrupt Macro offset,num
          Lea    DX, offset             ; Interrupt Routine Offset
          Mov    AL, num                ;           Number
          Mov    AH, 25H                ; Set
          Int    21H                    ;   Interrupt Vector
          EndM

Terminate_Process Macro rc
          ifdif  <rc>,<AL>
          ifdif  <rc>,<al>
          Mov    AL, rc                 ; Return Code
          endif
          endif
          Mov    AH, 4CH                ; Terminate
          Int    21H                    ;   Process
          EndM

.sall
          Page
          Code   Segment
          Assume CS:Code
          Org    100H


;----------------------- Miscellaneous Symbols -------------------------
Bios      Equ    40H                    ; Bios Data Segment
CR        Equ    0DH                    ; Carriage Return
Esc       Equ    1BH                    ; Escape
LF        Equ    0AH                    ; Line Feed
Sped_Off  Equ    4*2                    ; Offset to Speed Indicator
Stat_Off  Equ    10*2                   ; Offset to Status Indicators


;-------------------------- Hardware Ports -----------------------------
Kbd_Cntl  Equ    61H                    ; Keyboard Control Port
Tmr_Cntl  Equ    43H                    ; Timer Control Port
Tmr_Tone  Equ    42H                    ; Timer Tone Generator
          Page
;------------------------- Permanent Storage ---------------------------
Old_RTC   DD     0                      ; RTC Interrupt Vector
Old_Key   DD     0                      ; Keyboard Interrupt Vector
Old_Tmr   DD     0                      ; Timer Interrupt Vector

Prgm_Name DB     'ATTCLKFX Version 1.0 - $'
Prgm_LoGo DB     'AT&T PC 6300 Software Clock Fix'
Prgm_Lnth Equ    $-Prgm_Name            ; Program Name Length
          DB     CR,LF,'$'


;-------------------------- RTC Controls -------------------------------
Flags     DB     0                      ; Processing Flags
RTCFix    Equ    128                    ; RTC Fix Only
Active    Equ    64                     ; Window is Active
Graphics  Equ    32                     ; Video Graphics Mode
Chime     Equ    8                      ; Hourly Chime Enabled
Date      Equ    4                      ; Date Enabled
Time      Equ    2                      ; Time Enabled

RTC_Adjst DB     35                     ; RTC Adjustment Factor


;------------------------- RTC Work Areas ------------------------------
RTC_Skip  DB     35                     ; RTC Skip Counter


;-------------------------- Some Trickery ------------------------------
Trickery  Equ    $                      ; Start the
          Org    0100H                  ;   Trickery!!!
ATTCLKFX: Jmp    Initialize             ; Enter Initialization Routine
          Org    Trickery               ; End the Trickery
          Page
;-------------------------- RTC Interrupt ------------------------------
RTC_Interrupt:
          PushF                         ; Save What
          PushEm DS,ES,AX               ;   We Mess Up
          PushEm CS                     ; Point to
          PopEm  DS                     ;   Our Data

          Assume DS:Code

          Mov    AX, Bios               ; Point to
          Mov    ES, AX                 ;   BIOS Data


;-------------------------- Change the Hour ----------------------------
          Cmp    ES:[6CH], 65519        ; Timer Low at 65519
          Jne    RTC_Skip_A_Beat        ; No > Skip a Beat
          Mov    ES:[6CH], 65535        ; Force Next Hour
          Jmp    Short RTC_No_Skip      ; Set New Skip Time


;----------------------- Check for Skip A Beat -------------------------
RTC_Skip_A_Beat:
          Dec    RTC_Skip               ; Decrement Skip Counter
          Jnz    RTC_Exit               ; No > Invoke Old RTC
          Cmp    Word Ptr ES:[6CH], 0   ; RTC at Zero
          Je     RTC_No_Skip            ; Yes > Don't Skip
          Dec    Word Ptr ES:[6CH]      ; Skip a Timer Beat
RTC_No_Skip:
          Mov    AL, RTC_Adjst          ; Set New
          Mov    RTC_Skip, AL           ;   Skip Time


;---------------------- Invoke Old RTC Interrupt -----------------------
RTC_Exit:
          PopEm  AX,ES,DS               ; Restore What
          PopF                          ;   We Messed Up

          Jmp    CS:Old_RTC             ; Service Old RTC Interrupt

          Assume DS:Nothing

;--------------------- End of RTC Resident Code ------------------------
End_RTC_Front_End:
          Page
;-------------------- Keyboard and Timer Controls ----------------------
Controls  Equ    $                      ; Start of Front End Controls

PopUp_Key DW     1300H                  ; PopUp Key Code (Alt-R)
Dsply_Clr DB     07H                    ; Display Color
Wndw_Clr  DB     70H                    ; Window Color
Chime_Frq DW     2217                   ; Chime Frequency

Cntl_Lnth Equ    $-Controls             ; Length of Front End Controls


;------------------- Keyboard and Timer Work Areas ---------------------
Chime_Cnt DB     0

Clck_Size DW     0                      ; Size of on Screen Clock

Crnt_Hour DW     0                      ; Current Hour
Last_Hour DW     0                      ; Last Hour
Crnt_Sec  DW     0                      ; Current Second
Last_Sec  DW     0                      ; Last Second

Video_Col DW     0                      ; Columns Text & Attributes
Video_Csr DW     0                      ; Cursor Characteristics
Video_Seg DW     0                      ; Buffer Segment

Wndw_Off  DW     0                      ; Offset to Window Upper Left
Wndw_Rows DB     Mdl_Rows               ; Number of Window Rows
Wndw_Wdth DW     Mdl_Wdth               ; Width of Window Text


;------------------- Keyboard and Timer Constants ----------------------
DisAble   DB     'Dis'
EnAble    DB     ' En'


;-------------------------- KeyBoard Table -----------------------------
Key_Index DB     59,60,66,67,68         ; F1,F2,F8,F9,F10
Key_Cnt   Equ    $-Key_Index            ; Number of Table Entries
Key_Rtn   DW     Kbd_Faster             ; F1 - Faster
          DW     Kbd_Slower             ; F2 - Slower
          DW     Kbd_Date               ; F8  - Date Display
          DW     Kbd_Time               ; F9  - Time Display
Key_Last  DW     Kbd_Chime              ; F10 - Hourly Chime
          Page
;------------------------ Keyboard Interrupt ---------------------------
Keyboard_Interrupt:
          Sti                           ; Allow Interrupts
          Cmp    AH, 0                  ; Request for Key
          Jne    Kbd_Skip               ; No > Pass to Real Interrupt
          Test   CS:Flags, Active       ; Currently Active
          Jz     Kbd_FrontEnd           ; No > Front End the Request
Kbd_Skip:
          Jmp    DWord Ptr CS:Old_Key   ; Pass to BIOS Interrupt Rtn
Kbd_FrontEnd:
          PushF                         ; Simulate Real
          Call   DWord Ptr CS:Old_Key   ;   BIOS Interrupt Call
          Cmp    AX, CS:PopUp_Key       ; Pop-Up Key Code
          Jne    Kbd_IRet               ; No > Pass to Caller
          PushEm AX,BX,CX,DX,DI,SI,DS,ES
          PushF                         ; Save Flags
          Mov    AH, 0FH                ; Get Current
          Int    10H                    ;   Video Page
          Mov    AH, 03H                ; Get
          Int    10H                    ;   Cursor
          Mov    CS:Video_Csr, CX       ;     Characteristics
          Call   Cmn_Video              ; Set Video Controls
          Test   CS:Flags, Graphics     ; Graphics Mode
          Jz     Kbd_Activate           ; No > Activate FrontEnd
          PopF                          ; Restore Flags
          PopEm  ES,DS,SI,DI,DX,CX,BX,AX
Kbd_IRet:
          IRet                          ; Pass Key to Caller
Kbd_Activate:
          PopF                          ; Restore Flags
          Or     CS:Flags, Active       ; Show Active
          Call   Kbd_PopUp              ; PopUp the Window
          And    CS:Flags, 255-Active   ; Show Not Active
          PopEm  ES,DS,SI,DI,DX,CX,BX,AX
          Xor    AX, AX                 ; BIOS Key Read Function
          Jmp    Short Kbd_FrontEnd     ; Get New Key from BIOS


;------------------------- PopUp the Window ----------------------------
Kbd_PopUp:
          PushEm CS                     ; Set Data
          PopEm  DS                     ;   Segment
          PushEm CS                     ; Set Extra
          PopEm  ES                     ;   Segment

          Assume DS:Code, ES:Code

          Cld                           ; Set Forward Direction
          Mov    CX, Video_Csr          ; Cursor Characteristics
          And    CX, 0FFFH              ; Clear Cursor Mode
          Or     CX, 2000H              ; Set
          Mov    AH, 01H                ;   Invisible
          Int    10H                    ;     Cursor
          Call   Kbd_Flip_Flop          ; Display PopUp Window
Kbd_PopUp_Input:
          Xor    AX, AX                 ; Wait
          Int    16H                    ;   for Key
          Cmp    AL, Esc                ; Escape Key
          Je     Kbd_PopUp_Quit         ; Yes > Time to Quit
          Cmp    AX, PopUp_Key          ; Pop-Up Key Code
          Je     Kbd_PopUp_Quit         ; Yes > Time to Quit
          Call   Kbd_Window             ; Window Processing
          Jmp    Short Kbd_PopUp_Input  ; Wait for Another
Kbd_PopUp_Quit:
          Call   Kbd_Flip_Flop          ; Restore Screen
          Mov    CX, Video_Csr          ; Reset
          Mov    AH, 01H                ;   Cursor
          Int    10H                    ;     Mode
          Ret                           ;   and Return


;------------------------- Window Processing ---------------------------
Kbd_Window:
          Cmp    AL, 0                  ; Extended Scan Code
          Jz     Kbd_Window_Extended    ; Yes > Process the Key Stroke
Kbd_Window_Alarm:
          Mov    BL, 0                  ; Black on Black
          Mov    AL, 7                  ; Ring the Alarm
          Mov    AH, 0EH                ; Write
          Int    10H                    ;   Teletype
          Ret                           ;     and Return
Kbd_Window_Extended:
          Mov    CX, Key_Cnt            ; Number of Table Entries
          Lea    DI, Key_Index          ; Extended Code Table
          Xchg   AH, AL                 ; Extended Code
Repnz     ScaSb                         ;            in Table
          Jnz    Kbd_Window_Alarm       ; No > Sound Alarm
          Lea    DI, Key_Last           ; Point to Last Routine
          Shl    CX, 1                  ; Compute Extended
          Sub    DI, CX                 ;   Routine Offset
          Jmp    DS:[DI]                ; Invoke Extended Routine


;----------------------- Increase Clock Speed -------------------------
Kbd_Faster:                             ; F1 - Faster
          Mov    AL, 99                 ; Already
          Cmp    AL, RTC_Adjst          ;   Fastest Possible
          Je     Kbd_Window_Alarm       ; Yes > Sound Alarm
          Inc    RTC_Adjst              ; Increase Clock Speed
          Jmp    Short Kbd_Set_Speed    ; Show Speed Adjustment Factor


;----------------------- Decrease Clock Speed -------------------------
Kbd_Slower:                             ; F2 - Slower
          Mov    AL, 2                  ; Slowest
          Cmp    AL, RTC_Adjst          ;   Allowed
          Je     Kbd_Window_Alarm       ; Yes > Sound Alarm
          Dec    RTC_Adjst              ; Decrease Clock Speed


;------------------------- Set Clock Speed ----------------------------
Kbd_Set_Speed:
          Mov    AX, Video_Col          ; Number of Columns
          Mov    BX, Sped_Row           ; Compute
          Mul    BX                     ;   Location
          Sub    AX, Sped_Off           ;     of Speed
          Mov    DI, AX                 ;       Indicator
          Mov    BH, 1                  ; Supress Leading Zero
          Mov    AL, RTC_Adjst          ; Adjustment
          Call   Kbd_Set_Number         ;   Factor
          Ret                           ; Return


;----------------------- Toggle Date Status ---------------------------
Kbd_Date:                               ; F8 - Date Display
          Xor    Flags, Date            ; Toggle Date Flag
          Test   Flags, Date            ; Date Enabled
          Jnz    Kbd_Date_Set           ; Yes > Don't Blank Out
          Call   Kbd_Clear_Clock        ; Clear Window Clock
Kbd_Date_Set:
          Mov    AX, Video_Col          ; Number of Columns
          Mov    BX, Date_Row           ; Compute
          Mul    BX                     ;   Location
          Sub    AX, Stat_Off           ;     of Date
          Mov    DI, AX                 ;       Status Indicator
          Test   Flags, Date            ; Reset EnAbled/DisAbled Flag
          Jmp    Short Kbd_Toggle_Status  ; Set Status


;----------------------- Toggle Time Status ---------------------------
Kbd_Time:                               ; F9 - Time Display
          Xor    Flags, Time            ; Toggle Time Flag
          Test   Flags, Time            ; Time Enabled
          Jnz    Kbd_Time_Set           ; Yes > Don't Clear Window
          Call   Kbd_Clear_Clock        ; Clear Window Clock
Kbd_Time_Set:
          Mov    AX, Video_Col          ; Number of Columns
          Mov    BX, Time_Row           ; Compute
          Mul    BX                     ;   Location
          Sub    AX, Stat_Off           ;     of Time
          Mov    DI, AX                 ;       Status Indicator
          Test   Flags, Time            ; Reset EnAbled/DisAbled Flag
          Jmp    Short Kbd_Toggle_Status  ; Set Status


;----------------------- Toggle Chime Status --------------------------
Kbd_Chime:                              ; F10 - Hourly Chime
          Mov    AX, Crnt_Hour          ; Prevent
          Mov    Last_Hour, AX          ;   Erroneous Chime
          Xor    Flags, Chime           ; Toggle Chime Flag
          Mov    AX, Video_Col          ; Number of Columns
          Mov    BX, Chime_Row          ; Compute
          Mul    BX                     ;   Location
          Sub    AX, Stat_Off           ;     of Hourly Chime
          Mov    DI, AX                 ;       Status Indicator
          Test   Flags, Chime           ; Hourly Chime Enabled


;-------------------------- Toggle Status -----------------------------
Kbd_Toggle_Status:
          Lea    SI, DisAble            ; Assume DisAble
          Jz     Kbd_Toggle_Status_Dis  ; Yes > Show "Dis"
          Lea    SI, EnAble             ; Show " En"
Kbd_Toggle_Status_Dis:
          PushEm ES                     ; Save ES
          Mov    ES, Video_Seg          ; Video Segment
          Call   Kbd_Video_Off          ; Turn Video Off
          Mov    CX, 3                  ; 3 Characters
Kbd_Toggle_Status_Loop:
          MovSb                         ; Move " En" or "Dis"
          Inc    DI                     ; Skip the Attribute
          Loop   Kbd_Toggle_Status_Loop  ; Next Character
          Call   Kbd_Video_On           ; Turn Video On
          PopEm  ES                     ; Restore ES
          Ret                           ; Return


;----------------------- Clear Window Clock ---------------------------
Kbd_Clear_Clock:
          Mov    AH, Window+1           ; Attribute
          Mov    AL, ' '                ; Blank
          Mov    CX, Clck_Size          ; On Screen Clock Size
          Lea    DI, Window+(Mdl_Wdth*2)  ; Compute
          Sub    DI, CX                 ;     on Screen
          Sub    DI, CX                 ;       Clock Location
Rep       Stosw                         ; Clear on Screen Clock
          Ret                           ; Return


;---------------- Flip-Flop Window and Display Storage ----------------
Kbd_Flip_Flop:
          PushEm ES                     ; Save ES
          Call   Kbd_Video_Off          ; Turn Video Off
          Lea    SI, Window             ; Window Area
          Mov    DI, Wndw_Off           ; Offset to Window Upper Left
          Mov    ES, Video_Seg          ; Video Buffer Segment
          Mov    BL, Wndw_Rows          ; Number of Window Rows
Kbd_Flip_Flop_Next:
          Mov    CX, Wndw_Wdth          ; Window Width
Kbd_Flip_Flop_Row:
          Mov    AX, ES:[DI]            ; Swap Screen Word
          MovSw                         ;   With Window or
          Mov    [SI-2], AX             ;     Vice-Versa
          Loop   Kbd_Flip_Flop_Row      ; Complete the Row
          Add    DI, Wndw_Off           ; Next Window Row
          Dec    BL                     ; More Rows
          Jnz    Kbd_Flip_Flop_Next     ; Yes > Do All Rows
          Call   Kbd_Video_On           ; Turn Video On
          PopEm  ES                     ; Restore ES
          Ret                           ; Return


;----------------------- Set Number in Window ------------------------
Kbd_Set_Number:
          Mov    AH, 0                  ; Zero
          Mov    BL, 10                 ; Divide
          Div    BL                     ;   by Ten
          Add    AX, '00'               ; Convert to ASCII
          Cmp    BH, 0                  ; Suppress Leading Zero
          Jz     Kbd_Set_Number_Part    ; No > Set the Number
          Cmp    AL, '0'                ; Leading Zero
          Jnz    Kbd_Set_Number_Part    ; No > Set the Number
          Mov    AL, ' '                ; Set a Space
Kbd_Set_Number_Part:
          PushEm ES                     ; Save ES
          Call   Kbd_Video_Off          ; Turn Video Off
          Mov    ES, Video_Seg          ; Video Segment
          StoSb                         ; Set 1st Part
          Xchg   AH, AL                 ; Retrieve the Remainder
          Inc    DI                     ; Bump Past Attribute
          StoSb                         ; Set 2nd Part
          Call   Kbd_Video_On           ; Turn Video On
          PopEm  ES                     ; Restore ES
          Ret                           ; Return


;----------------------- Turn Video Off or On -------------------------
Kbd_Video_Off:
          PushEm AX,DX,ES               ; Save Registers
          Mov    AX, Bios               ; Bios Data
          Mov    ES, AX                 ;   Segment
          Mov    DX, ES:[63H]           ; Video 6845 Base
          Add    DX, 4                  ;   Compute Port Address
          Mov    AL, ES:[65H]           ; Current Video Setting
          And    AL, 0F7H               ; Turn Off
          Out    DX, AL                 ;   Video Enable Bit
          PopEm  ES,DX,AX               ; Restore Registers
          Ret                           ; Return

Kbd_Video_On:
          PushEm AX,DX,ES               ; Save Registers
          Mov    AX, Bios               ; Bios Data
          Mov    ES, AX                 ;   Segment
          Mov    DX, ES:[63H]           ; Video 6845 Base
          Add    DX, 4                  ;   Compute Port Address
          Mov    AL, ES:[65H]           ; Current Video Setting
          Or     AL, 08H                ; Turn On
          Out    DX, AL                 ;   Video Enable Bit
          PopEm  ES,DX,AX               ; Restore Registers
          Ret                           ; Return

;------------------- End of Keyboard Resident Code --------------------

          Assume DS:Nothing, ES:Nothing
          Page
;-------------------------- Timer Intercept ---------------------------
Timer_Interrupt:
          Sti                           ; Allow Interrupts
          PushF                         ; Save What
          PushEm AX,BX,CX,DX,DI,SI,DS,ES  ; We Mess Up
          Call   Cmn_Video              ; Set Video Controls

          PushEm CS                     ; Set Data
          PopEm  DS                     ;   Segment

          Assume DS:Code

;------------------------- Update Clock Time --------------------------
          Cld                           ; Forward Moves
          Mov    CX, ES:[6CH]           ; Timer Low
          Mov    AX, ES:[6EH]           ;   and High
          Mov    Crnt_Hour, AX          ; Current Hour
          PushEm CS                     ; Set Extra
          PopEm  ES                     ;   Segment

          Assume ES:Code

          Lea    DI, Clk_hh             ; Clock Hour
          Mov    BH, 1                  ; Leading Zero Suppression
          Call   Cmn_Set_Number         ; Store Hour

          Mov    AX, CX                 ; Timer Low
          Xor    DX, DX                 ; Compute
          Mov    CX, 1092               ;   Number
          Div    CX                     ;     of Minutes
          Lea    DI, Clk_mm             ; Clock Minute
          Mov    BH, 0                  ; No Leading Zero Suppression
          Call   Cmn_Set_Number         ; Store Minute

          Mov    AX, DX                 ; Remainder
          Mov    DX, 10                 ; Compute
          Mul    DX                     ;   Number
          Mov    CX, 182                ;     of
          Div    CX                     ;       Seconds
          Mov    Crnt_Sec, AX           ; Current Second
          Lea    DI, Clk_ss             ; Clock Second
          Mov    BH, 0                  ; No Leading Zero Suppression
          Call   Cmn_Set_Number         ; Store Second


;----------------------- Display Date and Time ------------------------
          Test   Flags, Graphics        ; Graphics Mode
          Jnz    Tmr_Hour_Chg           ; Yes > Check Hour Change
          Mov    AX, Crnt_Sec           ; Second
          Cmp    AX, Last_Sec           ;   Changed
          Jz     Tmr_Hour_Chg           ; No > Skip Display
          Mov    Last_Sec, AX           ; Last Second
          Mov    CX, 0000H              ; Show Nothing to Display
          Test   Flags, Active+Time     ; Window Active or Time EnAbled
          Jz     Tmr_Date               ; No > Check Date
          Lea    SI, Clk_Time           ; Clock Time
          Add    CX, Clk_TLnth          ; Set for Time Display
Tmr_Date:
          Test   Flags, Active+Date     ; Window Active or Date EnAbled
          Jz     Tmr_Dsply              ; No > Check for Display
          Lea    SI, Clk_Date           ; Clock Date
          Add    CX, Clk_DLnth          ; Set for Date Display
Tmr_Dsply:
          Cmp    CX, 0000H              ; Anything to Display
          Jz     Tmr_Hour_Chg           ; No > Check Hour Change
          Test   Flags, Active          ; Window Active
          Jz     Tmr_Dsply_Skip         ; Yes > Skip Setting Size
          Mov    Clck_Size, CX          ; Set Clock Size
Tmr_Dsply_Skip:
          Mov    AX, Video_Col          ; End of Screen
          Sub    AX, CX                 ; Compute
          Sub    AX, CX                 ;   Screen
          Mov    DI, AX                 ;     Position
          Call   Tmr_Display            ; Display Date and/or Time


;---------------------------- Hour Change ------------------------------
Tmr_Hour_Chg:
          Test   Flags, Chime           ; Hourly Chime Enabled
          Jz     Timer_Exit             ; No > Skip Chime
          Mov    AX, Crnt_Hour          ; Has the
          Cmp    AX, Last_Hour          ;   Hour Changed
          Jz     Timer_Exit             ; No > Skip Chime
          Call   Tmr_Chime              ; Ring the Hourly Chime


;---------------------- Exit the Timer Interrupt -----------------------
Timer_Exit:
          PopEm  ES,DS,SI,DI,DX,CX,BX,AX  ; Restore What
          PopF                          ;     We Messed Up

          Jmp    CS:Old_Tmr             ; And service old timer interrupt.


;----------------------- Display Date or Time ------------------------
Tmr_Display:
          PushEm ES                     ; Save ES
          Mov    AX, Bios               ; Bios Data
          Mov    ES, AX                 ;   Segment
          Mov    DX, ES:[63H]           ; Video 6845 Base
          Add    DX, 6                  ;   Compute Status Port Address
          Mov    ES, Video_Seg          ; Video Segment
          Mov    BH, Dsply_Clr          ; Display Color
Tmr_Display_Next:
          Lodsb                         ; Character
          Mov    BL, AL                 ;   to Display
Tmr_Display_Low:
          In     AL, DX                 ; Get Status
          Test   AL, 1                  ; Low
          Jnz    Tmr_Display_Low        ; No > Wait for Low
          Cli                           ; Interrupts Not Allowed
Tmr_Display_High:
          In     AL, DX                 ; Get Status
          Test   AL, 1                  ; High
          Jz     Tmr_Display_High       ; No > Wait for High
          Mov    AX, BX                 ; Character
          Stosw                         ;   and Attribute
          Sti                           ; Interrupts Allowed
          Loop   Tmr_Display_Next       ; Next Position
          PopEm  ES                     ; Restore ES
          Ret                           ; Return


;--------------------------- Hourly Chime ----------------------------
Tmr_Chime:
          Cmp    Chime_Cnt, 14*2        ; On and Off 14 Times
          Je     Tmr_Chime_Reset        ; Yes > Reset the Chime
          Cmp    Chime_Cnt, 0           ; First Time
          Jne    Tmr_Chime_Toggle       ; No > Toggle the Chime
          Mov    AL, 10110110B          ; Tell Timer Control
          Out    Tmr_Cntl, AL           ;   to Load the Tone Generator
          Mov    BX, Chime_Frq          ; Chime Frequency
          Mov    DX, 12H                ; Compute
          Xor    AX, AX                 ;   Tone
          Div    BX                     ;     to Generate
          Out    Tmr_Tone, AL           ; Update the
          Mov    AL, AH                 ;   Timer Tone
          Out    Tmr_Tone, AL           ;     Generator
          In     AL, Kbd_Cntl           ; Turn the
          And    AL, 11111100B          ;   Speaker
          Out    Kbd_Cntl, AL           ;     Off
Tmr_Chime_Toggle:
          In     AL, Kbd_Cntl           ; Toggle the
          Xor    AL, 00000011B          ;   Speaker
          Out    Kbd_Cntl, AL           ;     On and Off
          Inc    Chime_Cnt              ; Increment Chime Counter
          Ret                           ; Return
Tmr_Chime_Reset:
          In     AL, Kbd_Cntl           ; Keyboard Control Port
          And    AL, 11111100B          ; Turn Off
          Out    Kbd_Cntl, AL           ;   Speaker
          Mov    Chime_Cnt, 0           ; Reset Chime Counter
          Mov    AX, Crnt_Hour          ; Show New
          Mov    Last_Hour, AX          ;   Last Hour
          Ret                           ; Return

;-------------------- End of Timer Resident Code ---------------------

          Assume DS:Nothing, ES:Nothing
          Page
;-------------------- Common Resident Subroutines --------------------

;----------------------- SetUp Video Controls -------------------------
Cmn_Video:
          Mov    AX, Bios               ; Set BIOS
          Mov    ES, AX                 ;   Segment
          Mov    AX, ES:[49H]           ; Video Mode and Size
          Mov    BL, AL                 ; Video Mode
          Mov    CL, 7                  ; Compute Column Width
          Shr    AX, CL                 ;   Including
          Mov    CS:Video_Col, AX       ;     Attributes
          Sub    AX, CS:Wndw_Wdth       ; Compute Offset
          Sub    AX, CS:Wndw_Wdth       ;   To Window
          Mov    CS:Wndw_Off, AX        ;     Upper Left
          And    CS:Flags, 255-Graphics  ; Assume Text Mode
          Mov    AX, 0B000H             ; Mono Video Buffer Segment
          Cmp    BL, 7                  ; Mono Adapter
          Je     Cmn_Video_Set          ; Yes > Set Video Buffer Segment
          Mov    AX, 0B800H             ; Color Video Buffer Segment
          Cmp    BL, 3                  ; Text Mode
          Jle    Cmn_Video_Set          ; Yes > Set Video Buffer Segment
          Or     CS:Flags, Graphics     ; Show Graphics
Cmn_Video_Set:
          Mov    CS:Video_Seg, AX       ; Set Video Buffer Segment
          Ret                           ;   and Return


;----------------------- Set Converted Number ------------------------
Cmn_Set_Number:
          Mov    AH, 0                  ; Zero
          Mov    BL, 10                 ; Divide
          Div    BL                     ;   by Ten
          Add    AX, '00'               ; Convert to ASCII
          Cmp    BH, 0                  ; Suppress Leading Zero
          Jz     Cmn_Set_Number_Part    ; No > Set the Number
          Cmp    AL, '0'                ; Leading Zero
          Jnz    Cmn_Set_Number_Part    ; No > Set the Number
          Mov    AL, ' '                ; Set a Space
Cmn_Set_Number_Part:
          StoSb                         ; Set 1st Part
          Xchg   AH, AL                 ; Retrieve the Remainder
          StoSb                         ; Set 2nd Part
          Ret                           ; Return

;----------------- End of Common Resident Subroutines ----------------
          Page
;--------------------------- Clock Buffer ----------------------------
Clock     DB     23 DUP(0)
          Org    Clock                  ; Redefine Clock
Clk_Date  DB     ' '                    ; Start of Clock Date
Clk_wkday DB     'day'                  ; Weekday
          DB     ' '                    ; Spacer
Clk_dd    DB     'dd'                   ; dd  of ddmmmccyy
Clk_mmm   DB     'mmm'                  ; mmm of ddmmmccyy
Clk_cc    DB     'cc'                   ; cc  of ddmmmccyy
Clk_yy    DB     'yy'                   ; yy  of ddmmmccyy
Clk_DLnth Equ    $-Clk_Date             ; Clock Date Buffer Length
Clk_Time  DB     ' '                    ; Start of Clock Time
Clk_hh    DB     'hh'                   ; hh of hh:mm:ss
          DB     ':'                    ; Colon
Clk_mm    DB     'mm'                   ; mm of hh:mm:ss
          DB     ':'                    ; Colon
Clk_ss    DB     'ss'                   ; ss of hh:mm:ss
Clk_TLnth Equ    $-Clk_Time             ; Clock Time Buffer Length
Clk_Lnth  Equ    $-Clock                ; Clock Buffer Length


;-------------------------- Window Buffer ----------------------------
Window    DB     31*9*2 Dup(0)
Wndw_Lnth Equ    $-Window               ; Window Buffer Length


;---------------------- End of Resident Code -------------------------
End_Front_Ends:
          Page
;--------------- Initialization Messages and Constants -----------------
Optn_Updt DB     'Options Updated'
          DB     CR,LF,'$'
Invld_Err DB     'Invalid Parameter Specified'
          DB     CR,LF,'$'
Cnflt_Err DB     'Conflicting Parameters Specified'
          DB     CR,LF,'$'


;---------------------------- Model Window -----------------------------
Model     DB     '         day ddmmmyyyy hh:mm:ss'
Mdl_Wdth  Equ    $-Model
Clck_Row  Equ    ($-Model)/Mdl_Wdth
          DB     '[ATTCLKFX V1.0]ͻ'
          DB     ' '
Mdl_F1    Equ    $-Model
          DB     'F1  - Faster    Adjustment '
          DB     ' '
Mdl_F2    Equ    $-Model
          DB     'F2  - Slower    Factor: '
Mdl_Fctr  DB     '35 '
Sped_Row  Equ    ($-Model)/Mdl_Wdth
          DB     'Ķ'
          DB     ' '
Mdl_F8    Equ    $-Model
          DB     'F8  - Date Display '
Mdl_Date  DB     'Disabled '
Date_Row  Equ    ($-Model)/Mdl_Wdth
          DB     ' '
Mdl_F9    Equ    $-Model
          DB     'F9  - Time Display '
Mdl_Time  DB     'Disabled '
Time_Row  Equ    ($-Model)/Mdl_Wdth
          DB     ' '
Mdl_F10   Equ    $-Model
          DB     'F10 - Hourly Chime '
Mdl_Chime DB     'Disabled '
Chime_Row Equ    ($-Model)/Mdl_Wdth
          DB     'ͼ'
Mdl_Rows  Equ    ($-Model)/Mdl_Wdth
Mdl_Lnth  Equ    $-Model


;---------------------- Initialization Work Areas ----------------------
Crnt_CS   DW     0                      ; Current  ATTCLKFX Segment
Prev_CS   DW     0                      ; Previous ATTCLKFX Segment

Init_Flag DB     0                      ; Initialization Flags
Previous  Equ    128                    ; Previous Load Found
          Page
;---------------------- Initialization Tables --------------------------

Month     DB     'JanFebMarAprMayJunJulAugSepOctNovDec'
WeekDay   DB     'SunMonTueWedThuFriSat'

;-------------------------- Table Mapping ------------------------------
;-------------| NOTE: All tables follow the same format.|---------------
.xall
          Param  EntryMap                 ; Generate Table Entry Mapping


;------------------------- Parameter Table -----------------------------
Param_Tbl Label  Byte                     ; Start of Parameter Table

.sall
          Param  altf10,7100H,Param_Altx  ; ******* Dependency *******
          Param  altf1,6800H,Param_Altx   ;
          Param  altf2,6900H,Param_Altx   ; Alt-F10 must precede
          Param  altf3,6A00H,Param_Altx   ; Alt-F1 and Alt-F1 through
          Param  altf4,6B00H,Param_Altx   ; Alt-F9 must precede Alt-F.
          Param  altf5,6C00H,Param_Altx   ;
          Param  altf6,6D00H,Param_Altx   ; If they are not ordered
          Param  altf7,6E00H,Param_Altx   ; properly they will not be
          Param  altf8,6F00H,Param_Altx   ; found.
          Param  altf9,7000H,Param_Altx   ;
          Param  alt1,7800H,Param_Altx    ; ******* Dependency *******
          Param  alt2,7900H,Param_Altx
          Param  alt3,7A00H,Param_Altx
          Param  alt4,7B00H,Param_Altx
          Param  alt5,7C00H,Param_Altx
          Param  alt6,7D00H,Param_Altx
          Param  alt7,7E00H,Param_Altx
          Param  alt8,7F00H,Param_Altx
          Param  alt9,8000H,Param_Altx
          Param  alt0,8100H,Param_Altx
          Param  alta,1E00H,Param_Altx
          Param  altb,3000H,Param_Altx
          Param  altc,2E00H,Param_Altx
          Param  altd,2000H,Param_Altx
          Param  alte,1200H,Param_Altx
          Param  altf,2100H,Param_Altx
          Param  altg,2200H,Param_Altx
          Param  alth,2300H,Param_Altx
          Param  alti,1700H,Param_Altx
          Param  altj,2400H,Param_Altx
          Param  altk,2500H,Param_Altx
          Param  altl,2600H,Param_Altx
          Param  altm,3200H,Param_Altx
          Param  altn,3100H,Param_Altx
          Param  alto,1800H,Param_Altx
          Param  altp,1900H,Param_Altx
          Param  altq,1000H,Param_Altx
          Param  altr,1300H,Param_Altx
          Param  alts,1F00H,Param_Altx
          Param  altt,1400H,Param_Altx
          Param  altu,1600H,Param_Altx
          Param  altv,2F00H,Param_Altx
          Param  altw,1100H,Param_Altx
          Param  altx,2D00H,Param_Altx
          Param  alty,1500H,Param_Altx
          Param  altz,2C00H,Param_Altx
          Param  chime,0000H,Param_Chime
          Param  date,0000H,Param_Date
          Param  display,0000H,Param_Display
          Param  factor,0000H,Param_Factor
          Param  fixonly,0000H,Param_FixOnly
          Param  freq,0000H,Param_Freq
          Param  nochime,0000H,Param_NoChime
          Param  nodate,0000H,Param_NoDate
          Param  notime,0000H,Param_NoTime
          Param  time,0000H,Param_Time
          Param  window,0000H,Param_Window
          Param  af10,7100H,Param_Altx  ; ****** Abreviations ******
          Param  af1,6800H,Param_Altx   ;
          Param  af2,6900H,Param_Altx   ; These are the valid abrev-
          Param  af3,6A00H,Param_Altx   ; iations for the above par-
          Param  af4,6B00H,Param_Altx   ; ameters. They must be
          Param  af5,6C00H,Param_Altx   ; located after the full form
          Param  af6,6D00H,Param_Altx   ; of the parameter.
          Param  af7,6E00H,Param_Altx   ;
          Param  af8,6F00H,Param_Altx   ; ****** Abreviations ******
          Param  af9,7000H,Param_Altx   ;
          Param  a1,7800H,Param_Altx    ; ******* Dependency *******
          Param  a2,7900H,Param_Altx    ; ********** AGAIN *********
          Param  a3,7A00H,Param_Altx    ;
          Param  a4,7B00H,Param_Altx    ; Alt-F10 must precede
          Param  a5,7C00H,Param_Altx    ; Alt-F1 and Alt-F1 through
          Param  a6,7D00H,Param_Altx    ; Alt-F9 must precede Alt-F.
          Param  a7,7E00H,Param_Altx    ;
          Param  a8,7F00H,Param_Altx    ; If they are not ordered
          Param  a9,8000H,Param_Altx    ; properly they will not be
          Param  a0,8100H,Param_Altx    ; found.
          Param  aa,1E00H,Param_Altx    ;
          Param  ab,3000H,Param_Altx    ; ******* Dependency *******
          Param  ac,2E00H,Param_Altx
          Param  ad,2000H,Param_Altx
          Param  ae,1200H,Param_Altx
          Param  af,2100H,Param_Altx
          Param  ag,2200H,Param_Altx
          Param  ah,2300H,Param_Altx
          Param  ai,1700H,Param_Altx
          Param  aj,2400H,Param_Altx
          Param  ak,2500H,Param_Altx
          Param  al,2600H,Param_Altx
          Param  am,3200H,Param_Altx
          Param  an,3100H,Param_Altx
          Param  ao,1800H,Param_Altx
          Param  ap,1900H,Param_Altx
          Param  aq,1000H,Param_Altx
          Param  ar,1300H,Param_Altx
          Param  as,1F00H,Param_Altx
          Param  at,1400H,Param_Altx
          Param  au,1600H,Param_Altx
          Param  av,2F00H,Param_Altx
          Param  aw,1100H,Param_Altx
          Param  ax,2D00H,Param_Altx
          Param  ay,1500H,Param_Altx
          Param  az,2C00H,Param_Altx
          Param  ch,0000H,Param_Chime
          Param  da,0000H,Param_Date
          Param  di,0000H,Param_Display
          Param  fa,0000H,Param_Factor
          Param  fi,0000H,Param_FixOnly
          Param  fr,0000H,Param_Freq
          Param  nc,0000H,Param_NoChime
          Param  nd,0000H,Param_NoDate
          Param  nt,0000H,Param_NoTime
          Param  ti,0000H,Param_Time
          Param  wi,0000H,Param_Window
.xall
          Param  TableEnd               ; Mark the End of the Table


;--------------------------- Color Table -------------------------------
Color_Tbl Label  Byte                   ; Start of Color Table

.sall
          Param  black,0000H,0000H
          Param  blue,1001H,0000H
          Param  green,2002H,0000H
          Param  cyan,3003H,0000H
          Param  red,4004H,0000H
          Param  magenta,5005H,0000H
          Param  brown,6006H,0000H
          Param  white,7007H,0000H
          Param  grey,8008H,0000H
          Param  lightblue,9009H,0000H
          Param  lightgreen,0A00AH,0000H
          Param  lightcyan,0B00BH,0000H
          Param  lightred,0C00CH,0000H
          Param  lightmagenta,0D00DH,0000H
          Param  yellow,0E00EH,0000H
          Param  highintensitywhite,0F00FH,0000H
.xall
          PARAM  TableEnd               ; Mark the End of the Table
.sall
          Page

          Assume DS:Code, ES:Code

;--------------------------- Initialization --------------------------
Initialize:
          Mov    Word Ptr Old_RTC, 0    ; Zero Out
          Mov    Word Ptr Old_RTC+2, 0  ;   the Trickery!!!

          Cld                           ; Forward Moves
          Display Prgm_Name             ; Identify
          Display Prgm_LoGo             ;   ATTCLKFX
          Call   Prev_Load              ; See if Loaded Already
          Test   Init_Flag, Previous    ; Previous Load Found
          Jz     Init_Param             ; No > Locate Parameters

          PushEm DS                     ; Save DS
          Mov    AX, CS:Prev_CS         ; Previous
          Mov    DS, AX                 ;   ATTCLKFX Segment
          Mov    AL, DS:Flags           ; Previous
          Mov    CS:Flags, AL           ;   Flags
          Mov    AL, DS:RTC_Adjst       ; Previous
          Mov    CS:RTC_Adjst, AL       ;   RTC Adjustment Factor
          Test   CS:Flags, RTCFix       ; RTC Fix Only
          Jnz    Init_Skip1             ; Yes > Skip the Rest
          Lea    DI, Controls           ; Destination
          Lea    SI, Controls           ;   Controls
          Mov    CX, Cntl_Lnth          ; Number of Characters to Move
Rep       MovSb                         ; Move the Controls
Init_Skip1:
          PopEm  DS                     ; Restore DS

Init_Param:
          Call   Parameters             ; Process the Parameters
          Jnc    Init_Cnflt             ; Valid > Check Conflicts
          Terminate_Process 1           ; Terminate ATTCLKFX

Init_Cnflt:
          Test   Flags, RTCFix          ; RTC Fix Only
          Jz     Init_Window            ; No > Initialize the Window
          Test   Flags, 255-RTCFix      ; Any Conflicts
          Jz     Init_FixOnly           ; No > RTC Fix Only
          Display Prgm_Name             ; ATTCLKFX
          Display Cnflt_Err             ;   Conflicting Parameters
          Terminate_Process 2           ; Terminate ATTCLKFX
Init_FixOnly:
          Jmp    Init_Update            ; Update ATTCLKFX


;----------------------- Initialize the Window -----------------------
Init_Window:
          Mov    AL, RTC_Adjst          ; RTC Adjustment
          Lea    DI, Mdl_Fctr           ;   Factor
          Mov    BH, 1                  ; Leading Zero Suppression
          Call   Cmn_Set_Number         ; Store Adjustment Factor
          Call   Set_Status             ; Set Dis/Enable Indicators
          Lea    DI, Window             ; Fill Window
          Lea    SI, Model              ;   With Model
          Mov    AH, Wndw_Clr           ;     and Window Color
          Mov    CX, Mdl_Lnth           ; Length of Model
Init_Window_Next:
          LodSb                         ; Set Window Text
          StoSw                         ;   and Attribute
          Loop   Init_Window_Next       ; Do All Columns and Rows

          Mov    AH, Dsply_Clr          ; Clock Display Color
          Mov    AL, ' '                ; Blank
          Lea    DI, Window             ; Top Row
          Mov    CX, Mdl_Wdth           ; Display Row Width
Rep       StoSw                         ; Set Clock Display Row

          Mov    AL, Dsply_Clr          ; Clock Display Color
          Lea    DI, Window+(2*Mdl_F1)+1  ; Point to "F1"
          Mov    CX, 3                  ; 3 Attributes
Init_Window_F1:
          StoSb                         ; Set Attribute
          Inc    DI                     ; Next
          Loop   Init_Window_F1         ;   Attribute

          Lea    DI, Window+(2*Mdl_F2)+1  ; Point to "F2"
          Mov    CX, 3                  ; 3 Attributes
Init_Window_F2:
          StoSb                         ; Set Attribute
          Inc    DI                     ; Next
          Loop   Init_Window_F2         ;   Attribute

          Lea    DI, Window+(2*Mdl_F8)+1  ; Point to "F8"
          Mov    CX, 3                  ; 3 Attributes
Init_Window_F8:
          StoSb                         ; Set Attribute
          Inc    DI                     ; Next
          Loop   Init_Window_F8         ;   Attribute

          Lea    DI, Window+(2*Mdl_F9)+1  ; Point to "F9"
          Mov    CX, 3                  ; 3 Attributes
Init_Window_F9:
          StoSb                         ; Set Attribute
          Inc    DI                     ; Next
          Loop   Init_Window_F9         ;   Attribute

          Lea    DI, Window+(2*Mdl_F10)+1  ; Point to "F10"
          Mov    CX, 3                  ; 3 Attributes
Init_Window_F10:
          StoSb                         ; Set Attribute
          Inc    DI                     ; Next
          Loop   Init_Window_F10        ;   Attribute

          Call   Set_Date               ; Current Date to Clock


;------------------- Initialize the Software Clock -------------------
          Mov    AH, 0FEH               ; Get Current
          Int    1AH                    ;   Time

          Cli                           ; No Interrupts Allowed

          Mov    AX, Bios               ; Point to BIOS
          Mov    ES, AX                 ;   Data Area
          Xor    AX, AX                 ; Set
          Mov    AL, CH                 ;  Current
          Mov    Word Ptr ES:[6EH], AX  ;    Hour
          Mov    Last_Hour, AX          ; No Ring Upon Install

          PushEm DX                     ; Save Seconds
          Mov    AL, CL                 ; Minutes
          Mov    BX, 60                 ;   Convert to
          Mul    BX                     ;     Seconds
          PopEm  DX                     ; Restore
          Mov    BL, DH                 ;   Seconds
          Add    AX, BX                 ;     and Total
          Mov    DX, 182                ; Convert
          Mul    DX                     ;   to
          Mov    BX, 10                 ;     Timer
          Div    BX                     ;       Units
          Mov    Word Ptr ES:[6CH], AX  ; Set Current Timer Units

          Sti                           ; Ok to Interrupt Now


;---------------------- Update Active ATTCLKFX -----------------------
Init_Update:
          Test   Init_Flag, Previous    ; Previous Load Found
          Jz     Init_First             ; No > First Time Install
          PushEm ES                     ; Save ES

          Cli                           ; Do Not Allow Interrupts

          Mov    AX, CS:Prev_CS         ; Previous
          Mov    ES, AX                 ;   ATTCLKFX Segment
          Mov    AL, Flags              ; New
          Mov    ES:Flags, AL           ;   Flags
          Mov    AL, RTC_Adjst          ; New
          Mov    ES:RTC_Adjst, AL       ;   RTC Adjustment Factor
          Test   Flags, RTCFix          ; RTC Fix Only
          Jnz    Init_Skip2             ; Yes > Skip the Rest
          Lea    DI, Controls           ; Destination
          Lea    SI, Controls           ;   Controls
          Mov    CX, Cntl_Lnth          ; Number of Characters to Move
Rep       MovSb                         ; Move the Controls
          Lea    DI, Clock              ; Display
          Lea    SI, Clock              ;   Clock
          Mov    CX, Clk_Lnth           ; Number of Characters to Move
Rep       MovSb                         ; Move the Display Clock
          Lea    DI, Window             ; PopUp
          Lea    SI, Window             ;   Window
          Mov    CX, Wndw_Lnth          ; Number of Characters to Move
Rep       MovSb                         ; Move the PopUp Window
Init_Skip2:
          Sti                           ; Allow Interrupts

          PopEm  ES                     ; Restore ES

          Display Prgm_Name             ; ATTCLKFX
          Display Optn_Updt             ;   Options Updated
          Terminate_Process 0           ; Terminate ATTCLKFX


;------------------------- Activate ATTCLKFX -------------------------
Init_First:
          Get_Interrupt 08H             ; Get RTC Vector
          Mov    Word Ptr Old_RTC, BX   ; Store Offset
          Mov    Word Ptr Old_RTC[2], ES  ; and Segment
          Set_Interrupt RTC_Interrupt, 08H  ; Set New Vector

          Test   Flags, RTCFix          ; RTC Fix Only
          Jz     Init_All               ; No > Activate All
          Keep_Process 0, End_RTC_Front_End  ; Terminate and Stay Resident

Init_All:
          Get_Interrupt 16H             ; Get Keyboard Vector
          Mov    Word Ptr Old_Key, BX   ; Store Offset
          Mov    Word Ptr Old_Key[2], ES  ; and Segment
          Set_Interrupt Keyboard_Interrupt, 16H  ; Set New Vector

          Get_Interrupt 1CH             ; Get Timer Vector
          Mov    Word Ptr Old_Tmr, BX   ; Store Offset
          Mov    Word Ptr Old_Tmr[2], ES  ; and Segment
          Set_Interrupt Timer_Interrupt, 1CH  ; Set New Vector

          Keep_Process 0, End_Front_Ends  ; Terminate and Stay Resident
          Page
;-------------------- Initialization SubRoutines ---------------------

;---------------- Search for Previous Load of ATTCLKFX ---------------
Prev_Load:
          PushEm ES                     ; Save ES
          Mov    AX, CS                 ; Current
          Mov    Crnt_CS, AX            ;   ATTCLKFX Segment
          Mov    AH, 52H                ; Locate     (undocumented )
          Int    21H                    ;   1st      (function call)
          Mov    AX, Word Ptr ES:[BX-2]  ;    MCB Segment
Prev_Load_Next:
          PushEm DS                     ; Save DS
          Mov    DS, AX                 ; Next MCB Segment
          Cmp    Byte Ptr DS:[0], 4DH   ; MCB Identifier
          Jne    Prev_Load_NFD          ; No > Previous Load Not Found
          Mov    AX, Word Ptr DS:[1]    ; PSP Segment
          Cmp    CS:Crnt_CS, AX         ; Our ATTCLKFX Segment
          Je     Prev_Load_NFD          ; Yes > Previous Load Not Found
          Mov    ES, AX                 ; PSP Segment
          Mov    AX, DS                 ; Next
          Add    AX, Word Ptr DS:[3]    ;   MCB
          Inc    AX                     ;    Segment
          PopEm  DS                     ; Restore DS
          Lea    DI, Prgm_Name          ; Point to
          Lea    SI, Prgm_Name          ;   Program Name
          Mov    CX, Prgm_Lnth          ; Compare Prgm_LEN Characters
RepE      CmpSb                         ; ATTCLKFX
          Jne    Prev_Load_Next         ; No > Loop to the End
          Or     CS:Init_Flag, Previous  ; Show Previous Load Found
          Mov    AX, ES                 ; Previous
          Mov    CS:Prev_CS, AX         ;   ATTCLKFX Segment
          Jmp    Short Prev_Load_Quit   ; Return
Prev_Load_NFD:
          PopEm  DS                     ; Restore DS
Prev_Load_Quit:
          PopEm  ES                     ; Restore ES
          Ret                           ; Return


;---------------------- Process the Parameters -----------------------
Parameters:
          Mov    AL, [DS:0080H]         ; Any
          Cmp    AL, 0                  ;   Paramters
          Je     Param_Return           ; No > Just Return

          Mov    SI, 0081H              ; Start of
          Mov    DI, 0081H              ;   Parameters
Param_Loop:
          Mov    AL, DS:[SI]            ; Current Character
          Cmp    AL, CR                 ; End of Parameters
          Je     Param_Last             ; Yes > Set Last Character
          Or     AL, ' '                ; Set Lower Case
          Cmp    AL, '0'                ; Numeric
          Jb     Param_Junk             ; No > Junk It
          Cmp    AL, '9'                ; Numeric
          Jbe    Param_Keep             ; Yes > Keep It
          Cmp    AL, 'a'                ; Alphabetic
          Jb     Param_Junk             ; No > Junk It
          Cmp    AL, 'z'                ; Alphabetic
          Ja     Param_Junk             ; No > Junk It
Param_Keep:
          Mov    Byte Ptr DS:[DI], AL   ; Current Character
          Inc    DI                     ; Next Output Character
Param_Junk:
          Inc    SI                     ; Next Input
          Jmp    Short Param_Loop       ;   Character
Param_Last:
          Mov    Byte Ptr DS:[DI], AL   ; Last Character
          Mov    SI, 0081H              ; Start of Parameters
Param_Next:
          Cmp    Byte Ptr ES:[SI], CR   ; End of Parameters
          Je     Param_Return           ; Yes > Return to Caller
          Lea    DI, Param_Tbl          ; Point to Parameter Table
          Call   Param_LookUp           ; Table Lookup
          Jc     Param_Error            ; Not Found - Error Return
          Add    SI, DS:[DI+Entry_PLn]  ; End of Parameter
          Call   Word Ptr DS:[DI+Entry_Rtn]  ; Entry Routine
          Jnc    Param_Next             ; Ok > Next Parameter
Param_Error:
          Display Prgm_Name             ; ATTCLKFX
          Display Invld_Err             ;   Parameter Error
          Stc                           ; Set Carry
          Ret                           ;   and Return
Param_Return:
          Clc                           ; Clear Carry
          Ret                           ;   and Return


;---------------------- Parameter Table LookUp -----------------------
Param_LookUp:
          PushEm DI,SI                  ; Save DI and SI
          Mov    CX, DS:[DI+Entry_PLn]  ; Entry Value Length
          Add    DI, Entry_Prm          ; Entry Value
Repz      CmpSb                         ; This the Right Entry
          Je     Param_LookUp_Hit       ; Yes > Return This Entry
          PopEm  SI,DI                  ; Restore SI and DI
          Add    DI, DS:[DI+Entry_Len]  ; Next Entry
          Mov    AL, DS:[DI]            ; This
          Cmp    AL, 255                ;   the End
          Jne    Param_LookUp           ; No > Loop to End of Table
          Stc                           ; Set Carry
          Ret                           ;   and Return
Param_LookUp_Hit:
          PopEm  SI,DI                  ; Restore SI and DI
          Clc                           ; Clear Carry
          Ret                           ;   and Return


;---------------------- Process ALTx Parameter -----------------------
Param_Altx:
          Mov    AX, DS:[DI+Entry_Wrd]  ; PopUp
          Mov    PopUp_Key, AX          ;   Scan Code
          Clc                           ; Clear Carry
          Ret                           ;   and Return


;---------------------- Process CHIME Parameter ----------------------
Param_Chime:
          Or     Flags, Chime           ; Hourly Chime Wanted
          Jmp    Short Param_Chime_Ret  ; Return
Param_NoChime:
          And    Flags, 255-Chime       ; Hourly Chime Not Wanted
Param_Chime_Ret:
          Clc                           ; Clear Carry
          Ret                           ;   and Return


;---------------------- Process DATE Parameter -----------------------
Param_Date:
          Or     Flags, Date            ; Date Wanted
          Jmp    Short Param_Date_Ret   ; Return
Param_NoDate:
          And    Flags, 255-Date        ; Date Not Wanted
Param_Date_Ret:
          Clc                           ; Clear Carry
          Ret                           ;   and Return


;-------------------- Process DISPLAY Parameter ----------------------
Param_Display:
          Lea    DI, Color_Tbl          ; Point to Color Table
          Call   Param_LookUp           ; Table Lookup
          Jc     Param_Display_Ret      ; Not Found - Return Error
          Add    SI, DS:[DI+Entry_PLn]  ; End of Parameter
          Mov    AL, DS:[DI+Entry_Lo]   ; Foreground
          Mov    Dsply_Clr, AL          ;   Color
          Lea    DI, Color_Tbl          ; Point to Color Table
          Call   Param_LookUp           ; Table Lookup
          Jc     Param_Display_Done     ; Not Found - Return Ok
          Add    SI, DS:[DI+Entry_PLn]  ; End of Parameter
          Mov    AL, DS:[DI+Entry_Hi]   ; Background
          Or     Dsply_Clr, AL          ;   Color
Param_Display_Done:
          Clc                           ; Clear Carry
Param_Display_Ret:
          Ret                           ;   and Return


;-------------------- Process FIXONLY Parameter ----------------------
Param_FixOnly:
          Or     Flags, RTCFix          ; RTC Fix Only
          Clc                           ; Clear Carry
          Ret                           ;   and Return


;--------------------- Process FACTOR Parameter ----------------------
Param_Factor:
          Call   Numeric                ; Adjustment Factor
          Cmp    AL, 2                  ;   Valid
          Jb     Param_Factor_Err       ; No > Factor Error
          Cmp    AL, 99                 ;   Valid
          Ja     Param_Factor_Err       ; No > Factor Error
          Mov    RTC_Adjst, AL          ; Adjustment Factor
          Clc                           ; Clear Carry
          Ret                           ;   and Return
Param_Factor_Err:
          Stc                           ; Set Carry
          Ret                           ;   and Return


;---------------------- Process FREQ Parameter -----------------------
Param_Freq:
          Call   Numeric                ; Frequency
          Cmp    AX, 55                 ;   Valid
          Jb     Param_Freq_Err         ; No > Frequency Error
          Cmp    AX, 26576              ;   Valid
          Ja     Param_Freq_Err         ; No > Frequency Error
          Mov    Chime_Frq, AX          ; Chime Frequency
          Clc                           ; Clear Carry
          Ret                           ;   and Return
Param_Freq_Err:
          Stc                           ; Set Carry
          Ret                           ;   and Return


;---------------------- Process TIME Parameter -----------------------
Param_Time:
          Or     Flags, Time            ; Time Wanted
          Jmp    Param_Time_Ret         ; Return
Param_NoTime:
          And    Flags, 255-Time        ; Time Not Wanted
Param_Time_Ret:
          Clc                           ; Clear Carry
          Ret                           ;   and Return


;--------------------- Process WINDOW Parameter ----------------------
Param_Window:
          Lea    DI, Color_Tbl          ; Point to Color Table
          Call   Param_LookUp           ; Table Lookup
          Jc     Param_Window_Ret       ; Not Found - Return Error
          Add    SI, DS:[DI+Entry_PLn]  ; End of Parameter
          Mov    AL, DS:[DI+Entry_Lo]   ; Foreground
          Mov    Wndw_Clr, AL           ;   Color
          Lea    DI, Color_Tbl          ; Point to Color Table
          Call   Param_LookUp           ; Table Lookup
          Jc     Param_Window_Done      ; Not Found - Return Ok
          Add    SI, DS:[DI+Entry_PLn]  ; End of Parameter
          Mov    AL, DS:[DI+Entry_Hi]   ; Background
          Or     Wndw_Clr, AL           ;   Color
Param_Window_Done:
          Clc                           ; Clear Carry
Param_Window_Ret:
          Ret                           ;   and Return


;----------------------- Process Numeric Value -----------------------
Numeric:
          Mov    AX, 0                  ; Zero AX and
          Mov    BX, AX                 ;   BX Registers
Numeric_Loop:
          Mov    BL, Byte Ptr DS:[SI]   ; Current Character
          Cmp    BL, 30H                ; Numeric
          Jb     Numeric_Ret            ; No > Just Return
          Cmp    BL, 39H                ; Numeric
          Ja     Numeric_Ret            ; No > Just Return
          And    BL, 0FH                ; Remove Zone
          Mov    CX, 10                 ; Accumulate
          Mul    CX                     ;   Numeric
          Add    AX, BX                 ;     Value
          Inc    SI                     ; Next Input
          Jmp    Short Numeric_Loop     ;   Character
Numeric_Ret:
          Clc                           ; Clear Carry Flag
          Ret                           ; Return


;---------------------- Set Date in the Clock ------------------------
Set_Date:
          Mov    AH, 2AH                ; Current
          Int    21H                    ;   Date
          PushEm CX                     ; Save Year

          Mov    CL, 3                  ; Locate
          Mul    CL                     ;   Three
          Lea    SI, WeekDay            ;     Character
          Add    SI, AX                 ;       Weekday
          Lea    DI, Clk_wkday          ; Clock Weekday
          Mov    CX, 3                  ; Move 3 Characters
Rep       MovSb                         ; Move Weekday

          Mov    AL, DL                 ; Day
          Lea    DI, Clk_dd             ; Clock Day
          Mov    BH, 0                  ; No Leading Zero
          Call   Cmn_Set_Number         ; Place in Clock

          Mov    AL, DH                 ; Month
          Dec    AL                     ;   Relative to 0
          Mov    CL, 3                  ; Locate
          Mul    CL                     ;   Three
          Lea    SI, Month              ;     Character
          Add    SI, AX                 ;       Month
          Lea    DI, Clk_mmm            ; Clock Month
          Mov    CX, 3                  ; Move 3 Characters
Rep       MovSb                         ; Move Month

          PopEm  AX                     ; Restore Year
          Mov    CX, 100                ; Split the Century
          Xor    DX, DX                 ;   from the Year
          Div    CX                     ; AX = Century  DX = Year
          Lea    DI, Clk_cc             ; Clock Century
          Mov    BH, 0                  ; Allow Leading Zero
          Call   Cmn_Set_Number         ; Place in Clock
          Mov    AX, DX                 ; Year
          Lea    DI, Clk_yy             ; Clock Year
          Call   Cmn_Set_Number         ; Place in Clock
          Ret                           ; Return


;----------------------- Set Dis/EnAble Status -----------------------
Set_Status:
          PushEm SI,CX                  ; Save What We Mess Up
          Lea    DI, Mdl_Date           ; Date Status
          Lea    SI, EnAble             ; Show Enabled
          Test   Flags, Date            ; Date Wanted
          Jnz    Set_Status_Date        ; Yes > Show Enabled
          Lea    SI, DisAble            ; Show Disabled
Set_Status_Date:
          Call   Status_Set             ; Set the Status
          Lea    DI, Mdl_Time           ; Time Status
          Lea    SI, EnAble             ; Show Enabled
          Test   Flags, Time            ; Time Wanted
          Jnz    Set_Status_Time        ; Yes > Show Enabled
          Lea    SI, DisAble            ; Show Disabled
Set_Status_Time:
          Call   Status_Set             ; Set the Status
          Lea    DI, Mdl_Chime          ; Hourly Chime Status
          Lea    SI, EnAble             ; Show Enabled
          Test   Flags, Chime           ; Hourly Chime Wanted
          Jnz    Set_Status_Chime       ; Yes > Show Enabled
          Lea    SI, DisAble            ; Show Disabled
Set_Status_Chime:
          Call   Status_Set             ; Set the Status
          PopEm  CX,SI                  ; Restore What We Messed Up
          Ret                           ; Return

Status_Set:
          Mov    CX, 3                  ; Move 3 Bytes
Rep       MovSb                         ; Move En or Dis
          Ret                           ; Return

;------------------------ The End of ATTCLKFX -------------------------

          Code   EndS
          End    ATTCLKFX
